@@ -732,14 +732,21 @@ func TestInteractiveTerminal(t *testing.T) {
 			srv := httptest.NewServer(build.Session.Mux())
 			defer srv.Close()
 
-			u := url.URL{Scheme: "ws", Host: srv.Listener.Addr().String(), Path: build.Session.Endpoint + "/exec"}
-			conn, resp, err := websocket.DefaultDialer.Dial(u.String(), http.Header{"Authorization": []string{build.Session.Token}})
+			u := url.URL{
+				Scheme: "ws",
+				Host:   srv.Listener.Addr().String(),
+				Path:   build.Session.Endpoint + "/exec",
+			}
+			headers := http.Header{
+				"Authorization": []string{build.Session.Token},
+			}
+			conn, resp, err := websocket.DefaultDialer.Dial(u.String(), headers)
 			assert.NoError(t, err)
 			assert.Equal(t, c.expectedStatusCode, resp.StatusCode)
 
 			defer func() {
 				if conn != nil {
-					defer conn.Close()
+					conn.Close()
 				}
 			}()
 
